using System;
using System.Text;
using System.Windows.Forms;

using SdlDotNet;
using Tao.OpenGl;
using Tao.Platform.Windows;

namespace Jumpers
{
	/// <summary>
	/// Functionality for drawing text to the screen.
	/// </summary>
	public sealed class Fonter
	{
		private static int g_fontbase = 0;
		private static Gdi.GLYPHMETRICSFLOAT[] g_gmf = new Gdi.GLYPHMETRICSFLOAT[256];

		/// <summary>
		/// Initializes the fonter.
		/// </summary>
		public static void Init()
		{
			g_fontbase = Gl.glGenLists( 256 );

			// Create the font object
			IntPtr font = Gdi.CreateFont(
				-24,
				0,
				0,
				0,
				Gdi.FW_BOLD,
				false,
				false,
				false,
				Gdi.ANSI_CHARSET,
				Gdi.OUT_TT_PRECIS,
				Gdi.CLIP_DEFAULT_PRECIS,
				Gdi.ANTIALIASED_QUALITY,
				Gdi.FF_DONTCARE | Gdi.DEFAULT_PITCH,
				"Courier New" );

			// Generate a temporary form and generate the font with it
			using ( Form temp_form = new Form() )
			{
				IntPtr hdc = User.GetDC( temp_form.Handle );

				IntPtr oldfont = Gdi.SelectObject( hdc, font );
//				Wgl.wglUseFontBitmaps( hdc, 0, 255, g_fontbase );
				Wgl.wglUseFontOutlines( hdc, 0, 255, g_fontbase, 0, 0.0f, Wgl.WGL_FONT_POLYGONS, g_gmf );
				Gdi.SelectObject( hdc, oldfont );
				Gdi.DeleteObject( font );

				temp_form.Close();
			}
		}

		/// <summary>
		/// Writes text to the screen.
		/// </summary>
		/// <param name="x">The x coordinate.</param>
		/// <param name="y">The y coordinate.</param>
		/// <param name="size">The font size.</param>
		/// <param name="text">The text to write.</param>
		public static void Write( int x, int y, float size, String text )
		{
			byte[] data = Encoding.ASCII.GetBytes( text );

			float[] color = new float[ 4 ];
			Gl.glGetFloatv( Gl.GL_CURRENT_COLOR, color );
			float shadow = 0.3f;

			Gl.glPushMatrix();
			Gl.glLoadIdentity();
			Gl.glTranslatef( x, y, 0 );
			Gl.glScalef( size, -size, size );

			Gl.glRasterPos2f( 0, 0 );

			Gl.glPushAttrib( Gl.GL_LIST_BIT );
			Gl.glListBase( g_fontbase );

			Gl.glPushMatrix();
			Gl.glTranslatef( 0.1f, -0.1f, 0.0f );
//			Gl.glColor4f( color[ 0 ] * shadow, color[ 1 ] * shadow, color[ 2 ] * shadow, color[ 3 ] * shadow );
			Gl.glColor4f( 0, 0, 0, color[ 3 ] * shadow );
			Gl.glCallLists( text.Length, Gl.GL_UNSIGNED_BYTE, data );
			Gl.glPopMatrix();

			Gl.glColor4fv( color );
			Gl.glCallLists( text.Length, Gl.GL_UNSIGNED_BYTE, data );

			Gl.glPopAttrib();

			Gl.glPopMatrix();
		}
	}
}
