using System;
using System.Collections;

using SdlDotNet;
using Tao.OpenGl;

namespace Jumpers
{
	/// <summary>
	/// The main menu screen.
	/// </summary>
	public class MenuScreen : IScreen
	{
		private bool m_mouse_down = false;

		private ArrayList m_jumpers = new ArrayList();
		private int m_jumper_pause;

		public MenuScreen()
		{
		}

		public void Update( Mouse mouse )
		{
			bool is_down = mouse.IsButtonPressed( MouseButton.PrimaryButton );
			if ( !is_down && m_mouse_down )
			{ // Start the game
				MainLoop.Instance.GameScreen = new GameScreen();
			}
			m_mouse_down = is_down;

			// Add a background jumper?
			if ( m_jumper_pause-- <= 0 )
			{
				m_jumper_pause = 5;

				Jumper jumper = new Jumper();
				jumper.type = JumperType.White;
				jumper.pos.X = -200 + (float)Resources.random.NextDouble() * 700;
				jumper.pos.Y = -50;

				m_jumpers.Add( jumper );
			}

			// Update the jumpers
			Trampoline trampoline = new Trampoline();
			trampoline.pos.X = -999;
			int idx = 0;
			while ( idx < m_jumpers.Count )
			{
				Jumper jumper = (Jumper) m_jumpers[ idx ];
				jumper.Update( trampoline );

				if ( jumper.pos.Y > 480 )
					m_jumpers.RemoveAt( idx );
				else
					++idx;
			}
		}

		public void Draw()
		{
			// Draw the background
			Gl.glBegin( Gl.GL_TRIANGLE_STRIP );
			Gl.glColor3f( 0.0f, 0.4f, 0.6f );
			Gl.glVertex2f( 0, 0 );

			Gl.glColor3f( 0.0f, 0.2f, 0.3f );
			Gl.glVertex2f( 0, 480 );

			Gl.glColor3f( 0.0f, 0.4f, 0.6f );
			Gl.glVertex2f( 640, 0 );

			Gl.glColor3f( 0.0f, 0.2f, 0.3f );
			Gl.glVertex2f( 640, 480 );
			Gl.glEnd();

			// Draw the jumpers
			foreach ( Jumper jumper in m_jumpers )
			{
				jumper.Draw( new Vector2( -999, -999 ) );
			}

			// Draw the text
			Gl.glColor4f( 1.0f, 1.0f, 1.0f, 0.7f );
			Gl.glBlendFunc( Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA );

			Fonter.Write( 40, 120, 150.0f, "Jumpers" );

			float size = 17;
			int x = 100;
			int y = 220;
			int h = (int) size;
			Fonter.Write( x, y, size, "Save the jumping dudes by moving the platform" ); y += h;
			Fonter.Write( x, y, size, "under them before they hit the ground." ); y += h;
			Fonter.Write( x, y, size, "Use the spotlight to find the falling dudes." ); y += h;
			Fonter.Write( x, y, size, "The spotlight will pulse when a new dude jumps." ); y += h;
			y += h * 2;
			Fonter.Write( x, y, size, "Move the mouse to move the spotlight." ); y += h;
			Fonter.Write( x, y, size, "Use the arrow keys to move the platform." ); y += h;
			
			Fonter.Write( 100, 430, 40.0f, "Click mouse to start" );

			Gl.glColor4f( 1.0f, 1.0f, 1.0f, 0.35f );
			Fonter.Write( 220, 440, 10.0f, "(press esc anytime to quit)" );

			Gl.glColor4f( 1.0f, 1.0f, 1.0f, 0.2f );
			Fonter.Write( 8, 475, 8.0f, "2005  Jere Sanisalo" );

			Gl.glBlendFunc( Gl.GL_ONE, Gl.GL_ZERO );
		}

		public void KeyDown( Key key )
		{
		}

		public void KeyUp( Key key )
		{
		}
	}
}
