using System;
using System.Drawing;

namespace Jumpers
{
	/// <summary>
	/// 2D vector.
	/// </summary>
	public struct Vector2
	{
		public float X;
		public float Y;

		public static Vector2 Empty
		{
			get { return new Vector2( 0, 0 ); }
		}

		public static Vector2 operator + ( Vector2 a, Vector2 b )
		{
			return new Vector2( a.X + b.X, a.Y + b.Y );
		}

		public static Vector2 operator - ( Vector2 a, Vector2 b )
		{
			return new Vector2( a.X - b.X, a.Y - b.Y );
		}

		public static Vector2 operator - ( Vector2 a )
		{
			return new Vector2( -a.X, -a.Y );
		}

		public static Vector2 operator * ( Vector2 a, float v )
		{
			return new Vector2( a.X * v, a.Y * v );
		}

		public static Vector2 operator / ( Vector2 a, float v )
		{
			return new Vector2( a.X / v, a.Y / v );
		}

		public Vector2( float x, float y )
		{
			this.X = x;
			this.Y = y;
		}

		public Vector2( Point p )
			: this( p.X, p.Y )
		{
		}

		public Vector2( PointF p )
			: this( p.X, p.Y )
		{
		}

		public float DotProduct( Vector2 a )
		{
			return ( X * a.X + Y * a.Y );
		}

		public float Norm()
		{
			return DotProduct( this );
		}

		public float Abs()
		{
			return (float) Math.Sqrt( Norm() );
		}

		public void Normalize()
		{
			float len = Abs();
			if ( len > 0.0001f )
			{
				X /= len;
				Y /= len;
			}
			else
			{
				X = Y = 0.0f;
			}
		}

		public static float Distance( Vector2 a, Vector2 b )
		{
			return ( b - a ).Abs();
		}
	}
}
